/*
** main.c
**
** Nokia 3310 Example Program
** Target: ATMEGA :: AVR-GCC
**
** Written by Tony Myatt - 2007
** Quantum Torque - www.quantumtorque.com
*/
#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <avr/pgmspace.h>
#include <stdio.h>
#include <stdlib.h>
#include "lcd.h"

#define BUFF_SIZE_X      (LCD_X_RES)
#define BUFF_SIZE_Y      (LCD_Y_RES / 8)

#define STANGA		0
#define DREAPTA		1
#define SUS			2
#define JOS			3
#define USOR        0
#define GREU		1
#define ATINS		1

typedef struct directie {
	int x;
	int y;
} directie;

static directie snake[85];
static directie mancare;
static directie bonus;
static int dim = 4;
static int btn_apasat = STANGA;
static int nivel = -1;
static int scor = 0;
static int col_mancare = 0;
static int final = 0;

void initializare_butoane() {
	DDRC = 0xff;
    PORTC = 0x00;
	
	/* Setez pinii PC ca intrare. */
	DDRC &= ~((1 << PC0) | (1 << PC1) | (1 << PC5) | (1 << PC4) | (1 << PC3) | (1 << PC2));

	/* Activez rezistenta de pull-up pentru pinii PC. */
	PORTC |= (1 << PC0) | (1 << PC1) | (1 << PC5) | (1 << PC4) | (1 << PC3) | (1 << PC2);
 	_delay_ms(150);
}

void initializare_joc(void) {
	int i;
	lcd_goto_xy(5,2);
	lcd_str("SNAKE");
	lcd_goto_xy(2,4);
	lcd_str("Iulia Rascol");
	lcd_goto_xy(9,5);
	lcd_str("333CA");
	
	for(i = 0; i < 6; i++) {
		_delay_ms(200);
	}
}

void speaker_morse(int tip)
{
    int delay = 1;
    int i;
    DDRD |= (1 << PD5);     // PD5 trebuie sa fie output

    for (i = 0; i < tip * 50 / delay; i++)
    {
        PORTD ^= (1 << PD5); // alternam starea iesirii
        _delay_ms(1);
    }
}

void countdown(void) {

	_delay_ms(1000);
	lcd_clear();
	lcd_goto_xy(7,3);
	lcd_str("3");

	_delay_ms(1000);

	lcd_clear();
	lcd_goto_xy(7,3);
	lcd_str("2");
	_delay_ms(1000);

	lcd_clear();
	lcd_goto_xy(7,3);
	lcd_str("1");
	_delay_ms(1000);

	lcd_clear();
	lcd_goto_xy(5,3);
	lcd_str("Start!");
	
	//sunet pt inceputul jocului
	speaker_morse(20);
	_delay_ms(5);
	
	lcd_clear();
}

void start_joc(void) {
	if (nivel == -1) {
		lcd_clear();
		lcd_goto_xy(1,3);
		lcd_str("Alege nivelul!");
		_delay_ms(300);
	}

	if ((PINC & (1 << PC1)) == 0){
		nivel = USOR;
		lcd_clear();
		_delay_ms(5);
		lcd_goto_xy(3,3);
		lcd_str("EASY PEASY!");
		_delay_ms(200);
		countdown();
		_delay_ms(200);
	} else
	if ((PINC & (1 << PC0)) == 0){
		nivel = GREU;
		lcd_clear();
		_delay_ms(5);
		lcd_goto_xy(4,3);
		lcd_str("HARDCORE!");
		_delay_ms(200);
		countdown();
		_delay_ms(200);
	}
}

void initializare_snake(void) {
	int i;
	
	mancare.x = rand() % 14 + 1;
	mancare.y = rand() % 7 + 1;
	
	bonus.x = -1;
	bonus.y = -1;

	for (i = 0; i < dim; i++) {
		snake[i].y = 4;
		snake[i].x = 5 + i;
	}
}

void coliziune_mancare (void) {
	if(mancare.x == snake[0].x && mancare.y == snake[0].y && col_mancare == 0) {
		dim++;
		scor++;
		col_mancare = ATINS;
		_delay_ms(5);
	}
}

void coliziune_bonus (void) {
	if(bonus.x == snake[0].x && bonus.y == snake[0].y) {
		dim += 2;
		scor += 3;
		bonus.x = -1;
		bonus.y = -1;
		lcd_goto_xy(4, 3);
		lcd_str("SUPER!");
		_delay_ms(10);
	}
}

void coliziune_snake (void) {
	int i;
	for(i = 2; i < dim; i++) {
		if(snake[0].x == snake[i].x && snake[0].y == snake[i].y) {
			speaker_morse(35);
			_delay_ms(5);
			final = 1;
			break;
		}
	}	
}

void coliziune_perete(void) {
	if (snake[0].y == 1 || snake[0].y == 6 || (scor % 3 == 0 && scor != 0 && snake[0].x == 1)) {
	 	speaker_morse(35);
		_delay_ms(5);
		final = 1;
	}
}

void sfarsit() {
	lcd_clear();
	lcd_goto_xy(1, 3);
	lcd_str("C`est fini ...");
	
	char *punctaj = (char*)malloc(sizeof(char) * 3);
	itoa(scor, punctaj, 10);
	lcd_goto_xy(4, 4);
	lcd_str(punctaj);
	lcd_goto_xy(6, 4);
	if(scor == 1)
		lcd_str(" punct");
	else
		lcd_str(" puncte");
}

void desenare_fereastra() {
	int i;

	if(col_mancare == ATINS) {
		if(nivel == USOR) {
			mancare.x = rand() % 10 + 1;
			mancare.y = rand() % 5 + 1;
		} else
		if(nivel == GREU) {
			mancare.x = rand() % 7 + 2;
			mancare.y = rand() % 3 + 3;
		}
		col_mancare = 0;
		_delay_ms(5);
	}
	
	if(nivel == GREU) {
		lcd_goto_xy(1, 1);
		lcd_str("##############");
		lcd_goto_xy(1, 6);
		lcd_str("##############");
		
		if(scor % 3 == 0 && scor != 0) {
			lcd_goto_xy(1, 1);
			lcd_str("#");
			lcd_goto_xy(1, 2);
			lcd_str("#");
			lcd_goto_xy(1, 3);
			lcd_str("#");
			lcd_goto_xy(1, 4);
			lcd_str("#");
			lcd_goto_xy(1, 5);
			lcd_str("#");
			lcd_goto_xy(1, 6);
			lcd_str("#");
		}
	}
	
	lcd_goto_xy(mancare.x, mancare.y);
	lcd_chr('x');
	
	if(scor % 5 != 0) {
		bonus.y = rand() % 7 + 1;
		bonus.x = rand() % 14 + 1;
	}
	
	if(scor % 5 == 0 && scor != 0) {
		lcd_goto_xy(bonus.x, bonus.y);
		lcd_chr('$');
	}
		
	for (i = 0; i < dim; i++) {
		lcd_goto_xy(snake[i].x, snake[i].y);
		lcd_chr('*');
	}
}

void miscare_sarpe(void) {
	int i;
	
	// Verificare butoane apasate 
    // STANGA
    if ((PINC & (1 << PC5)) == 0 && btn_apasat != DREAPTA) {
        btn_apasat = STANGA;
    } else
    
    // DREAPTA
    if ((PINC & (1 << PC2)) == 0 && btn_apasat != STANGA) {
        btn_apasat = DREAPTA;
    } else

	// JOS
    if ((PINC & (1 << PC4)) == 0 && btn_apasat != SUS) {
        btn_apasat = JOS;
    } else
    
    // SUS
    if ((PINC & (1 << PC3)) == 0 && btn_apasat != JOS) {
        btn_apasat = SUS;
    }
	
	//schimbare la fiecare frame
	if(btn_apasat == DREAPTA) {
		snake[0].x = snake[0].x % 14 + 1;
	} else	
	if(btn_apasat == SUS) {
		if(snake[0].y == 1) {
			snake[0].y = 6;
		} else {
			snake[0].y --;
		}
	} else 
	if(btn_apasat == STANGA) {
		if(snake[0].x == 1) {
			snake[0].x = 14;
		} else {
			snake[0].x --;
		}
	} else 
	if(btn_apasat == JOS) {
		snake[0].y = snake[0].y % 7 + 1;	
	}
	
	for (i = dim - 1; i > 0; i--) {
		snake[i].x = snake[i - 1].x;
		snake[i].y = snake[i - 1].y;
	}
	
	desenare_fereastra();
}


/* main -- Program starts here */
int main(void)
{	
	// Setup LCD
	lcd_init();
	lcd_contrast(0x40);
	
	initializare_butoane();	
	initializare_snake();
	initializare_joc();
	_delay_ms(100);

	while(1) {
		int i;
		start_joc();
		
		if(scor % 5 == 0) {
			coliziune_bonus();
		}
		
		coliziune_mancare();
		
		if(nivel == USOR || nivel == GREU) {
			miscare_sarpe();
		}
			
		if(final == 0) {
			coliziune_snake();
			if(nivel == GREU) {
				coliziune_perete();
			}
			if(final == 1) {
				sfarsit();
				_delay_ms(3000);
				lcd_clear();
				lcd_goto_xy(3, 3);
				lcd_str("Reiei jocul?");				
				final = 0;
				nivel = -1;
				dim = 4;
				scor = 0;
				snake[0].x = 6;
				snake[0].y = 3;
				for (i = 1; i < dim; i++) {
					snake[i].y = 3;
					snake[i].x = 6 - i;
				}
				mancare.x = 5;
				mancare.y = 3;
				btn_apasat = DREAPTA;
				_delay_ms(3000);
				if ((PINC & (1 << PC1)) == 0) {
					lcd_clear();
					lcd_goto_xy(6, 3);
					lcd_str("BYE!");
					_delay_ms(50);
					break;
				} 
			}
		}
		
		_delay_ms(150);
		lcd_clear();
	}
}
